<?php

/**
 * @file
 * Contains Drush commands for CMS Content Sync.
 */

use Drupal\cms_content_sync\Entity\Pool;
use Drupal\cms_content_sync\Entity\Flow;
use Drupal\cms_content_sync\SyncCorePoolExport;
use Drupal\cms_content_sync\SyncCoreFlowExport;
use Symfony\Component\Console\Input\InputOption;

/**
 * Implements hook_drush_command().
 */
function cms_content_sync_drush_command() {
  $items['content-sync-export'] = [
    'description' => dt('Export configuration to the Sync Core.'),
    'aliases' => ['cse'],
    'options' => [
      'force' => 'Whether to ignore that another site is already using the same site ID. Useful if you change the URL of a site.',
    ],
  ];

  $items['content-sync-login'] = [
    'description' => dt('Export configuration to the Sync Core.'),
    'aliases' => ['csl'],
  ];

  $items['content-sync-pull'] = [
    'description' => dt('Pull entities from Sync Core.'),
    'aliases' => ['csp'],
    'options' => [
      'flow_id' => 'The flow the entities should be pulled for.',
    ],
    'examples' => [
      'drush csp' => 'Pull all entities from all flows',
      'drush csp --flow_id="example_flow"' => 'Pull all entities from the "example_flow".',
    ],
  ];

  return $items;
}

/**
 * Export configuration to the Content Sync backend.
 *
 * @options force Whether to ignore that another site is already using the
 *     same site ID. Useful if you change the URL of a site.
 *
 * @throws \Exception
 */
function drush_cms_content_sync_content_sync_export($force = FALSE) {
  $force = drush_get_option('force', FALSE);
  drush_print('Started export of pools.');
  foreach (Pool::getAll() as $pool) {
    $exporter = new SyncCorePoolExport($pool);

    if (!$force && $exporter->siteIdExists()) {
      throw new \Exception('Another site is already using this site ID for the pool ' . $pool->id);
    }

    $steps = $exporter->prepareBatch();
    foreach ($steps as $step) {
      $exporter->executeBatch($step);
    }
  }
  drush_print('Finished export of pools.');

  drush_print('Started export of flows.');
  foreach (Flow::getAll() as $flow) {
    $exporter = new SyncCoreFlowExport($flow);
    $steps    = $exporter->prepareBatch();
    foreach ($steps as $step) {
      $exporter->executeBatch($step);
    }
  }
  drush_print('Finished export of flows.');
}

/**
 * Kindly ask the Sync Core to login again.
 */
function drush_cms_content_sync_content_sync_login() {
  drush_print('Calling /login for all connections.');

  $result = [];
  foreach (Flow::getAll() as $flow) {
    $exporter = new SyncCoreFlowExport($flow);
    $result = array_merge($result, $exporter->login());
  }

  foreach ($result as $url => $success) {
    if (!$success) {
      $url = preg_replace('%https?://(.*)@(.*)$%', '$2', $url);
      drush_print('FAILED to login from ' . $url);
    }
  }

  drush_print('Finished login.');
}

/**
 * Pull entities from the Sync Core.
 *
 * @param array $options
 */
function drush_cms_content_sync_content_sync_pull($flow_id = '') {

  $flow_id = drush_get_option('flow_id', '');
  $flow_id = empty($flow_id) ? NULL : $flow_id;

  $flows = Flow::getAll();
  $client = \Drupal::httpClient();

  foreach ($flows as $id => $flow) {
    if($flow_id && $id!=$flow_id) {
      continue;
    }

    $exporter = new SyncCoreFlowExport($flow);

    $result = $exporter->startSync(FALSE);

    if(empty($result)) {
      drush_print('No automated import configured for Flow: '.$flow->label());
      continue;
    }

    drush_print('Started pull of entities for Flow: '.$flow->label());

    foreach($result as $url=>$response) {
      if(!$response) {
        drush_print('> Failed to pull! Sync URL: '.$url);
        continue;
      }

      if(empty($response['total'])) {
        drush_print('> Nothing to do for: '.$url);
        continue;
      }

      $url .= '/synchronize/'.$response['id'].'/status';

      $goal = $response['total'];
      $progress = 0;

      while($progress < $goal) {
        if($progress>0) {
          sleep(5);
        }

        $body = json_decode($client->get($url)->getBody(),TRUE);

        $progress = $body['processed'];
        if($progress==$goal) {
          drush_print('> Pulled '.$goal.' entities.');
        }
        elseif($progress==0) {
          sleep(5);
        }
        else {
          drush_print('Pulled '.$progress.' of '.$goal.' entities: '.floor($progress/$goal*100).'%');
        }
      }
    }
  }
}
