<?php

namespace Drupal\cms_content_sync\Commands;

use Drupal\cms_content_sync\Entity\Flow;
use Drupal\cms_content_sync\Entity\Pool;
use Drupal\cms_content_sync\SyncCoreFlowExport;
use Drupal\cms_content_sync\SyncCorePoolExport;
use Drush\Commands\DrushCommands;
use Symfony\Component\Console\Input\InputOption;

/**
 * CMS Content Sync Drush Commands.
 */
class CMSContentSyncCommands extends DrushCommands {

  /**
   * Export configuration to the Sync Core.
   *
   * @command cms_content_sync:content-sync-export
   * @aliases cse
   * @options force Whether to ignore that another site is already using the
   *     same site ID. Useful if you change the URL of a site.
   *
   * @throws \Exception
   */
  public function content_sync_export($options = ['force' => FALSE]) {
    $this->output()->writeln('Started export of pools.');
    foreach (Pool::getAll() as $pool) {
      $exporter = new SyncCorePoolExport($pool);

      if (!$options['force'] && $exporter->siteIdExists()) {
        throw new \Exception('Another site is already using the site ID "' . $pool->getSiteId() . '" for the pool "' . $pool->id . '"');
      }

      $steps = $exporter->prepareBatch();
      foreach ($steps as $step) {
        $exporter->executeBatch($step);
      }
    }
    $this->output()->writeln('Finished export of pools.');
    $this->output()->writeln('Started export of flows.');
    foreach (Flow::getAll() as $flow) {
      $exporter = new SyncCoreFlowExport($flow);
      $steps    = $exporter->prepareBatch();
      foreach ($steps as $step) {
        $exporter->executeBatch($step);
      }
    }
    $this->output()->writeln('Finished export of flows.');
  }

  /**
   * Kindly ask the Sync Core to login again.
   *
   * @command cms_content_sync:content-sync-login
   * @aliases csl
   */
  public function content_sync_login() {
    $this->output()->writeln('Calling /login for all connections.');

    $result = [];
    foreach (Flow::getAll() as $flow) {
      $exporter = new SyncCoreFlowExport($flow);
      $result = array_merge($result, $exporter->login());
    }

    foreach ($result as $url => $success) {
      if (!$success) {
        $url = preg_replace('%https?://(.*)@(.*)$%', '$2', $url);
        $this->output()->writeln('FAILED to login from ' . $url);
      }
    }

    $this->output()->writeln('Finished login.');
  }

  /**
   * Kindly ask the Sync Core to pull all entities.
   *
   * @param array $options
   *
   * @command cms_content_sync:content-sync-pull
   * @aliases csp
   * @options flow_id The flow the entities should be pulled from.
   * @usage cms_content_sync:content-sync-pull
   *   Pulls all entities from all flows.
   * @usage cms_content_sync:content-sync-pull --flow_id='example_flow'
   *   Pulls all entities from the example flow.
   */
  public function content_sync_pull($options = ['flow_id'=> InputOption::VALUE_REQUIRED]) {

    $flow_id = empty($options['flow_id']) ? NULL : $options['flow_id'];

    $flows = Flow::getAll();
    $client = \Drupal::httpClient();

    foreach ($flows as $id => $flow) {
      if($flow_id && $id!=$flow_id) {
        continue;
      }

      $exporter = new SyncCoreFlowExport($flow);

      $result = $exporter->startSync(FALSE);

      if(empty($result)) {
        $this->output()->writeln('No automated import configured for Flow: '.$flow->label());
        continue;
      }

      $this->output()->writeln('Started pull of entities for Flow: '.$flow->label());

      foreach($result as $url=>$response) {
        if(!$response) {
          $this->output()->writeln('> Failed to pull! Sync URL: '.$url);
          continue;
        }

        if(empty($response['total'])) {
          $this->output()->writeln('> Nothing to do for: '.$url);
          continue;
        }

        $url .= '/synchronize/'.$response['id'].'/status';

        $goal = $response['total'];
        $progress = 0;

        while($progress < $goal) {
          if($progress>0) {
            sleep(5);
          }

          $body = json_decode($client->get($url)->getBody(),TRUE);

          $progress = $body['processed'];
          if($progress==$goal) {
            $this->output()->writeln('> Pulled '.$goal.' entities.');
          }
          elseif($progress==0) {
            sleep(5);
          }
          else {
            $this->output()->writeln('Pulled '.$progress.' of '.$goal.' entities: '.floor($progress/$goal*100).'%');
          }
        }
      }
    }
  }

}
